Package: radiant.data
Title: Data Menu for Radiant: Business Analytics using R and Shiny
Version: 1.6.8
Date: 2025-9-15
Authors@R: c(
    person("Vincent", "Nijs", email = "radiant@rady.ucsd.edu", role = c("aut", "cre")),
    person("Niklas", "von Hertzen", email = "niklasvh@gmail.com", role = c("aut"), comment = "html2canvas library")
  )
Description: The Radiant Data menu includes interfaces for loading, saving,
    viewing, visualizing, summarizing, transforming, and combining data. It also
    contains functionality to generate reproducible reports of the analyses
    conducted in the application.
Depends: R (>= 4.3.0), magrittr (>= 1.5), ggplot2 (>= 3.4.2), lubridate
        (>= 1.7.4), tidyr (>= 0.8.2), dplyr (>= 1.1.2)
Imports: tibble (>= 1.4.2), rlang (>= 0.4.10), broom (>= 0.5.2), car
        (>= 3.0-0), knitr (>= 1.20), markdown (>= 1.7), rmarkdown(>=
        2.22), shiny (>= 1.8.1), jsonlite (>= 1.0), shinyAce (>=
        0.4.1), psych (>= 1.8.4), DT (>= 0.28), readr (>= 1.1.1),
        readxl (>= 1.0.0), writexl (>= 0.2), scales (>= 0.4.0), curl
        (>= 2.5), rstudioapi (>= 0.7), import (>= 1.1.0), plotly (>=
        4.7.1), glue (>= 1.3.0), shinyFiles (>= 0.9.1), stringi (>=
        1.2.4), randomizr (>= 0.20.0), patchwork (>= 1.0.0), bslib (>=
        0.5.0), png, MASS, base64enc
Suggests: arrow (>= 12.0.1), dbplyr (>= 2.1.1), DBI (>= 0.7), RSQLite
        (>= 2.0), RPostgres (>= 1.4.4), webshot (>= 0.5.0), testthat
        (>= 2.0.0), pkgdown (>= 1.1.0)
URL: https://github.com/radiant-rstats/radiant.data/,
        https://radiant-rstats.github.io/radiant.data/,
        https://radiant-rstats.github.io/docs/
BugReports: https://github.com/radiant-rstats/radiant.data/issues/
License: AGPL-3 | file LICENSE
LazyData: true
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2025-09-16 06:31:33 UTC; vnijs
Author: Vincent Nijs [aut, cre],
  Niklas von Hertzen [aut] (html2canvas library)
Maintainer: Vincent Nijs <radiant@rady.ucsd.edu>
Repository: CRAN
Date/Publication: 2025-09-16 07:00:02 UTC
Built: R 4.6.0; ; 2026-05-01 00:24:02 UTC; unix
