# Generated by roxygen2: do not edit by hand

S3method(dtab,data.frame)
S3method(dtab,explore)
S3method(dtab,pivotr)
S3method(plot,character)
S3method(plot,pivotr)
S3method(render,character)
S3method(render,datatables)
S3method(render,plotly)
S3method(render,shiny.render.function)
S3method(store,character)
S3method(store,explore)
S3method(store,pivotr)
S3method(summary,explore)
S3method(summary,pivotr)
export(Search)
export(add_class)
export(add_description)
export(arrange_data)
export(as_character)
export(as_distance)
export(as_dmy)
export(as_dmy_hm)
export(as_dmy_hms)
export(as_duration)
export(as_factor)
export(as_hm)
export(as_hms)
export(as_integer)
export(as_mdy)
export(as_mdy_hm)
export(as_mdy_hms)
export(as_numeric)
export(as_tibble)
export(as_ymd)
export(as_ymd_hm)
export(as_ymd_hms)
export(bs_theme)
export(center)
export(choose_dir)
export(choose_files)
export(ci_label)
export(ci_perc)
export(combine_data)
export(combinedata)
export(copy_all)
export(copy_attr)
export(copy_from)
export(cv)
export(date)
export(deregister)
export(describe)
export(does_vary)
export(dtab)
export(empty_level)
export(explore)
export(filter_data)
export(filterdata)
export(find_dropbox)
export(find_gdrive)
export(find_home)
export(find_project)
export(fixMS)
export(fix_names)
export(fix_smart)
export(flip)
export(format_df)
export(format_nr)
export(formatdf)
export(formatnr)
export(get_class)
export(get_data)
export(get_summary)
export(getclass)
export(getdata)
export(getsummary)
export(ggplotly)
export(glance)
export(glue)
export(glue_collapse)
export(glue_data)
export(indexr)
export(install_webshot)
export(inverse)
export(is.empty)
export(is_double)
export(is_not)
export(is_numeric)
export(is_string)
export(iterms)
export(knit_print)
export(kurtosi)
export(launch)
export(level_list)
export(ln)
export(load_clip)
export(make_arrange_cmd)
export(make_train)
export(make_vec)
export(max_rm)
export(me)
export(mean_rm)
export(median_rm)
export(meprop)
export(min_rm)
export(modal)
export(month)
export(mutate_ext)
export(n_missing)
export(n_obs)
export(normalize)
export(p01)
export(p025)
export(p05)
export(p10)
export(p25)
export(p75)
export(p90)
export(p95)
export(p975)
export(p99)
export(parse_path)
export(pcv)
export(pfun)
export(pivotr)
export(plot_annotation)
export(pmean)
export(pmedian)
export(pp01)
export(pp025)
export(pp05)
export(pp10)
export(pp25)
export(pp75)
export(pp95)
export(pp975)
export(pp99)
export(prop)
export(psd)
export(psum)
export(pvar)
export(qscatter)
export(qterms)
export(radiant.data)
export(radiant.data_url)
export(radiant.data_viewer)
export(radiant.data_window)
export(read_files)
export(refactor)
export(register)
export(render)
export(round_df)
export(rounddf)
export(rownames_to_column)
export(save_clip)
export(sd_rm)
export(sdpop)
export(sdprop)
export(se)
export(search_data)
export(seprop)
export(set_attr)
export(show_duplicated)
export(sig_stars)
export(skew)
export(slice_data)
export(square)
export(sshh)
export(sshhr)
export(standardize)
export(store)
export(subplot)
export(sum_rm)
export(table2data)
export(theme_version)
export(tibble)
export(tidy)
export(toFct)
export(to_fct)
export(var_rm)
export(varpop)
export(varprop)
export(view_data)
export(viewdata)
export(visualize)
export(wday)
export(weighted.sd)
export(which.pmax)
export(which.pmin)
export(wrap_plots)
export(writePNG)
export(write_parquet)
export(xtile)
import(dplyr)
import(ggplot2)
import(shiny)
importFrom(MASS,fractions)
importFrom(base64enc,dataURI)
importFrom(broom,glance)
importFrom(broom,tidy)
importFrom(bslib,bs_theme)
importFrom(bslib,theme_version)
importFrom(car,Recode)
importFrom(curl,curl_download)
importFrom(glue,glue)
importFrom(glue,glue_collapse)
importFrom(glue,glue_data)
importFrom(import,from)
importFrom(jsonlite,fromJSON)
importFrom(knitr,knit)
importFrom(knitr,knit2html)
importFrom(knitr,knit_print)
importFrom(lubridate,as.duration)
importFrom(lubridate,date)
importFrom(lubridate,dmy)
importFrom(lubridate,hm)
importFrom(lubridate,hms)
importFrom(lubridate,hour)
importFrom(lubridate,is.Date)
importFrom(lubridate,is.POSIXt)
importFrom(lubridate,mdy)
importFrom(lubridate,minute)
importFrom(lubridate,month)
importFrom(lubridate,now)
importFrom(lubridate,parse_date_time)
importFrom(lubridate,second)
importFrom(lubridate,wday)
importFrom(lubridate,week)
importFrom(lubridate,year)
importFrom(lubridate,ymd)
importFrom(lubridate,ymd_hms)
importFrom(magrittr,"%$%")
importFrom(magrittr,"%<>%")
importFrom(magrittr,"%T>%")
importFrom(magrittr,add)
importFrom(magrittr,divide_by)
importFrom(magrittr,extract2)
importFrom(magrittr,set_colnames)
importFrom(magrittr,set_names)
importFrom(magrittr,set_rownames)
importFrom(markdown,mark_html)
importFrom(patchwork,plot_annotation)
importFrom(patchwork,wrap_plots)
importFrom(plotly,ggplotly)
importFrom(plotly,renderPlotly)
importFrom(plotly,subplot)
importFrom(png,writePNG)
importFrom(psych,kurtosi)
importFrom(psych,skew)
importFrom(randomizr,block_ra)
importFrom(randomizr,complete_ra)
importFrom(readr,locale)
importFrom(readr,problems)
importFrom(readr,read_csv)
importFrom(readr,read_delim)
importFrom(readr,read_rds)
importFrom(readr,write_csv)
importFrom(readr,write_rds)
importFrom(readxl,read_excel)
importFrom(rlang,.data)
importFrom(rlang,parse_exprs)
importFrom(rmarkdown,html_dependency_bootstrap)
importFrom(rmarkdown,html_document)
importFrom(rmarkdown,pdf_document)
importFrom(rmarkdown,render)
importFrom(rmarkdown,word_document)
importFrom(rstudioapi,getActiveProject)
importFrom(rstudioapi,insertText)
importFrom(rstudioapi,isAvailable)
importFrom(rstudioapi,selectFile)
importFrom(shiny,getDefaultReactiveDomain)
importFrom(shiny,makeReactiveBinding)
importFrom(shiny,paneViewer)
importFrom(shiny,tags)
importFrom(shinyAce,aceEditor)
importFrom(shinyAce,updateAceEditor)
importFrom(shinyFiles,getVolumes)
importFrom(shinyFiles,parseDirPath)
importFrom(shinyFiles,parseFilePaths)
importFrom(shinyFiles,parseSavePath)
importFrom(shinyFiles,shinyFileChoose)
importFrom(shinyFiles,shinyFileSave)
importFrom(shinyFiles,shinyFilesButton)
importFrom(shinyFiles,shinyFilesLink)
importFrom(shinyFiles,shinySaveButton)
importFrom(shinyFiles,shinySaveLink)
importFrom(stats,IQR)
importFrom(stats,as.formula)
importFrom(stats,chisq.test)
importFrom(stats,dbinom)
importFrom(stats,density)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(stats,weighted.mean)
importFrom(stringi,stri_trans_general)
importFrom(tibble,as_tibble)
importFrom(tibble,rownames_to_column)
importFrom(tibble,tibble)
importFrom(tidyr,extract)
importFrom(tidyr,gather)
importFrom(tidyr,separate)
importFrom(tidyr,spread)
importFrom(tools,file_ext)
importFrom(utils,browseURL)
importFrom(utils,combn)
importFrom(utils,head)
importFrom(utils,install.packages)
importFrom(utils,read.table)
importFrom(utils,str)
importFrom(utils,tail)
importFrom(utils,write.table)
importFrom(writexl,write_xlsx)
